@@ -35,6 +35,14 @@ use cmdline::{AuthSubcommand, Command};
 
 pub const INSECURE_DIST_SERVER_TOKEN: &str = "dangerously_insecure_server";
 
+#[cfg(not(any(
+    all(target_os = "linux", target_arch = "x86_64"),
+    target_os = "freebsd"
+)))]
+fn main() {
+    compile_error!("Distributed compilation is only support on Linux/x86_64 and FreeBSD!");
+}
+
 // Only supported on x86_64 Linux machines and on FreeBSD
 #[cfg(any(
     all(target_os = "linux", target_arch = "x86_64"),
@@ -48,7 +56,10 @@ fn main() {
         .iter()
         .for_each(|incr_str| match env::var(incr_str) {
             Ok(incr_val) if incr_val == "1" => {
-                println!("sccache: increment compilation is  prohibited.");
+                println!(
+                    "sccache: incremental compilation is prohibited: Unset {} to continue.",
+                    incr_str
+                );
                 std::process::exit(1);
             }
             _ => (),
@@ -85,7 +96,7 @@ fn create_server_token(server_id: ServerId, auth_token: &str) -> String {
     format!("{} {}", server_id.addr(), auth_token)
 }
 fn check_server_token(server_token: &str, auth_token: &str) -> Option<ServerId> {
-    let mut split = server_token.splitn(2, |c| c == ' ');
+    let mut split = server_token.splitn(2, ' ');
     let server_addr = split.next().and_then(|addr| addr.parse().ok())?;
     match split.next() {
         Some(t) if t == auth_token => Some(ServerId::new(server_addr)),
@@ -172,9 +183,6 @@ fn run(command: Command) -> Result<i32> {
                     token_check::ValidJWTCheck::new(audience, issuer, &jwks_url)
                         .context("Failed to create a checker for valid JWTs")?,
                 ),
-                scheduler_config::ClientAuth::Mozilla { required_groups } => {
-                    Box::new(token_check::MozillaCheck::new(required_groups))
-                }
                 scheduler_config::ClientAuth::ProxyToken { url, cache_secs } => {
                     Box::new(token_check::ProxyTokenCheck::new(url, cache_secs))
                 }
@@ -225,6 +233,7 @@ fn run(command: Command) -> Result<i32> {
             builder,
             cache_dir,
             public_addr,
+            bind_address,
             scheduler_url,
             scheduler_auth,
             toolchain_cache_size,
@@ -289,6 +298,7 @@ fn run(command: Command) -> Result<i32> {
                 .context("Failed to create sccache server instance")?;
             let http_server = dist::http::Server::new(
                 public_addr,
+                bind_address,
                 scheduler_url.to_url(),
                 scheduler_auth,
                 server,
@@ -302,14 +312,22 @@ fn run(command: Command) -> Result<i32> {
 
 fn init_logging() {
     if env::var(sccache::LOGGING_ENV).is_ok() {
-        match env_logger::Builder::from_env(sccache::LOGGING_ENV).try_init() {
+        let mut builder = env_logger::Builder::from_env(sccache::LOGGING_ENV);
+
+        // Enable millisecond precision timestamps if SCCACHE_LOG_MILLIS is set
+        if env::var("SCCACHE_LOG_MILLIS").is_ok() {
+            builder.format_timestamp_millis();
+        }
+
+        match builder.try_init() {
             Ok(_) => (),
             Err(e) => panic!("Failed to initialize logging: {:?}", e),
         }
     }
 }
 
-const MAX_PER_CORE_LOAD: f64 = 10f64;
+// Maximum number of jobs per core - only occurs for one core, usually less, see load_weight()
+const MAX_PER_CORE_LOAD: f64 = 2f64;
 const SERVER_REMEMBER_ERROR_TIMEOUT: Duration = Duration::from_secs(300);
 const UNCLAIMED_PENDING_TIMEOUT: Duration = Duration::from_secs(300);
 const UNCLAIMED_READY_TIMEOUT: Duration = Duration::from_secs(60);
@@ -399,6 +417,20 @@ impl Default for Scheduler {
     }
 }
 
+fn load_weight(job_count: usize, core_count: usize) -> f64 {
+    // Oversubscribe cores just a little to make up for network and I/O latency. This formula is
+    // not based on hard data but an extrapolation to high core counts of the conventional wisdom
+    // that slightly more jobs than cores achieve the shortest compile time. Which is originally
+    // about local compiles and this is over the network, so be slightly less conservative.
+    let cores_plus_slack = core_count + 1 + core_count / 8;
+    // Note >=, not >, because the question is "can we add another job"?
+    if job_count >= cores_plus_slack {
+        MAX_PER_CORE_LOAD + 1f64 // no new jobs for now
+    } else {
+        job_count as f64 / core_count as f64
+    }
+}
+
 impl SchedulerIncoming for Scheduler {
     fn handle_alloc_job(
         &self,
@@ -415,7 +447,7 @@ impl SchedulerIncoming for Scheduler {
                 let mut best_load: f64 = MAX_PER_CORE_LOAD;
                 let now = Instant::now();
                 for (&server_id, details) in servers.iter_mut() {
-                    let load = details.jobs_assigned.len() as f64 / details.num_cpus as f64;
+                    let load = load_weight(details.jobs_assigned.len(), details.num_cpus);
 
                     if let Some(last_error) = details.last_error {
                         if load < MAX_PER_CORE_LOAD {
